import "Turbine.Gameplay";
import "Turbine.UI";
import "Turbine.Utils";
import "PengorosPlugins.BuffBars";

CoolDownList = class(Turbine.UI.ListBox);

function CoolDownList:Constructor(settings)
    Turbine.UI.ListBox.Constructor(self);
    
    self.settings = settings;
    self:SetOrientation(Turbine.UI.Orientation.Horizontal);
    self:SetMaxItemsPerLine(1);
    self:SetMouseVisible(false);
    
    -- hook cool down events
    coolDownManager:AddListener(self);
    
    -- setup the initial state
    local i;
    local pots = coolDownManager:GetPots();
    for i = 1, table.getn(pots) do
        self:PotAdded(pots[i]);
    end
end

function CoolDownList:Destroy()
    coolDownManager:RemoveListener(self);
end

function CoolDownList:PotAdded(pot)
    -- is this pot type filtered
    local category = pot:GetCategory();
    if((category == Settings.DiseaseCoolDown) and (self.settings.showDiseaseCD == false)) then return
    elseif((category == Settings.FearCoolDown) and (self.settings.showFearCD == false)) then return
    elseif((category == Settings.PoisonCoolDown) and (self.settings.showPoisonCD == false)) then return
    elseif((category == Settings.WoundCoolDown) and (self.settings.showWoundCD == false)) then return
    elseif((category == Settings.MoraleCoolDown) and (self.settings.showMoraleCD == false)) then return
    elseif((category == Settings.PowerCoolDown) and (self.settings.showPowerCD == false)) then return
    end

    -- create the cool down item
    local coolDownItem = CoolDownItem(self.settings);
    coolDownItem:SetPot(pot);
    coolDownItem:SetSize(self.settings.barWidth, self.settings.barHeight);
    
    -- sort methods
    if (self.settings.sortMethod == EffectWindow.SortTime) then
        local coolDownEndTime = pot:GetEndTime();

        local i = 0;
        local insertAt = -1;

        for i = 1, self:GetItemCount() do
            local testPot = self:GetItem(i):GetPot();
            local testPotEndTime = testPot.startTime + testPot.duration;
            local dif = testPotEndTime - coolDownEndTime;
            if (not self.settings.sortAscending) then
                dif = dif * -1;
            end
            if (self.settings.fillBottomToTop) then
                dif = dif * -1;
            end
            
            if (dif >= 0) then
                insertAt = i;
                break;
            end
        end

        if (insertAt == -1) then
            self:AddItem(coolDownItem);
        else
            self:InsertItem(insertAt, coolDownItem);
        end
    elseif (self.settings.sortMethod == EffectWindow.SortName) then
        local coolDownName = coolDown.name;
        
        local i = 0;
        local insertAt = -1;

        for i = 1, self:GetItemCount() do
            local testPot = self:GetItem(i):GetPot();
            local testPotName = testPot.name;
            
            local dif = -1;
            if (string.lower(coolDownName) < string.lower(testPotName)) then
                dif = 1;
            end
            if (not self.settings.sortAscending) then
                dif = dif * -1;
            end
            if (self.settings.fillBottomToTop) then
                dif = dif * -1;
            end
            
            if (dif >= 0) then
                insertAt = i;
                break;
            end
        end

        if (insertAt == -1) then
            self:AddItem(coolDownItem);
        else
            self:InsertItem(insertAt, coolDownItem);
        end
    else
        -- no sorting
        self:AddItem(coolDownItem);
    end
end

function CoolDownList:PotRemoved(pot)
    local i;

    -- remove the list item
    for i = 1, self:GetItemCount() do
        local potListItem = self:GetItem(i):GetPot();

        if (pot == potListItem) then
            local potElement = self:GetItem(i);
            potElement:SetVisible(false);
            self:RemoveItemAt(i);
            break;
        end
    end
end
