import "Turbine.Gameplay";
import "Turbine.UI";
import "Turbine.Utils";
import "PengorosPlugins.BuffBars";

EffectList = class(Turbine.UI.ListBox);

function EffectList:Constructor(debuffList, settings)
    Turbine.UI.ListBox.Constructor(self);
    
    self.settings = settings;
    
    self.debuffList = debuffList; -- is this a buff or debuff list
    self:SetOrientation(Turbine.UI.Orientation.Horizontal);
    self:SetMaxItemsPerLine(1);
    self:SetMouseVisible(false);
    
    -- hook effect events
    effectManager:AddListener(self);
    
    -- setup the initial state
    local i;
    local effects = effectManager:GetEffects();
    for i = 1, effects:GetCount() do
        self:EffectAdded(effects:Get(i));
    end
end

function EffectList:Destroy()
    effectManager:RemoveListener(self);
end

function EffectList:EffectsCleared()
    local count = self:GetItemCount();
    while (count > 0) do
        local item = self:GetItem(1);
        item:SetVisible(false);
        self:RemoveItemAt(1);
        count = count - 1;
    end
end

function EffectList:EffectAdded(effect)
    -- what type of effects are we tracking
    local debuff = effect:IsDebuff();
    if ((self.debuffList == true) and (debuff == false)) then return
    elseif ((self.debuffList == false) and (debuff == true)) then return
    end
    
    -- is the effect on the override list
    local name = effect:GetName();
    if (self.settings:IsOverrideEffect(name) == nil) then
        -- is this type filtered?
        local curable = effect:IsCurable();
        if ((self.settings.showCurableOnly == true) and (curable == false) and (debuff == true)) then return
        end
        
        -- is this category filtered
        local category = effect:GetCategory();
        if((category == Turbine.Gameplay.EffectCategory.Disease) and (self.settings.showDisease == false)) then return
        elseif((category == Turbine.Gameplay.EffectCategory.Fear) and (self.settings.showFear == false)) then return
        elseif((category == Turbine.Gameplay.EffectCategory.Wound) and (self.settings.showWound == false)) then return
        elseif((category == Turbine.Gameplay.EffectCategory.Poison) and (self.settings.showPoison == false)) then return
        elseif((category == Turbine.Gameplay.EffectCategory.Physical) and (self.settings.showPhysical == false)) then return
        elseif((category == Turbine.Gameplay.EffectCategory.Elemental) and (self.settings.showElemental == false)) then return
        elseif((category == Turbine.Gameplay.EffectCategory.Song) and (self.settings.showSong == false)) then return
        elseif((category == Turbine.Gameplay.EffectCategory.Corruption) and (self.settings.showCorruption == false)) then return
        elseif((category == Turbine.Gameplay.EffectCategory.Cry) and (self.settings.showCry == false)) then return
        elseif((category == Turbine.Gameplay.EffectCategory.Undefined) and (self.settings.showUndefined == false)) then return
        end
        
        -- is the duration filtered
        local duration = effect:GetDuration();
        if (self.settings.minDurationFilter and duration <= self.settings.minDuration) then return
        end
        if (self.settings.maxDurationFilter and duration >= self.settings.maxDuration) then return
        end

        -- is the name filtered
        if (self.settings:IsIgnoredEffect(name)) then return
        end
    end

    -- create the effect item
    local effectItem = EffectItem(self.settings);
    effectItem:SetEffect(effect);
    effectItem:SetSize(self.settings.barWidth, self.settings.barHeight);
    effectItem.RemoveEffect = function(effect)
        self:EffectRemoved(effect);
    end
    
    -- sort methods
    if (self.settings.sortMethod == EffectWindow.SortTime) then
        local effectEndTime = effect:GetStartTime() + effect:GetDuration();

        local i = 0;
        local insertAt = -1;

        for i = 1, self:GetItemCount() do
            local testEffect = self:GetItem(i):GetEffect();
            local testEffectEndTime = testEffect:GetStartTime() + testEffect:GetDuration();
            local dif = testEffectEndTime - effectEndTime;
            if (not self.settings.sortAscending) then
                dif = dif * -1;
            end
            if (self.settings.fillBottomToTop) then
                dif = dif * -1;
            end
            
            if (dif >= 0) then
                insertAt = i;
                break;
            end
        end

        if (insertAt == -1) then
            self:AddItem(effectItem);
        else
            self:InsertItem(insertAt, effectItem);
        end
    elseif (self.settings.sortMethod == EffectWindow.SortName) then
        local effectName = effect:GetName();
        
        local i = 0;
        local insertAt = -1;

        for i = 1, self:GetItemCount() do
            local testEffect = self:GetItem(i):GetEffect();
            local testEffectName = testEffect:GetName();
            
            local dif = -1;
            if (string.lower(effectName) < string.lower(testEffectName)) then
                dif = 1;
            end
            if (not self.settings.sortAscending) then
                dif = dif * -1;
            end
            if (self.settings.fillBottomToTop) then
                dif = dif * -1;
            end
            
            if (dif >= 0) then
                insertAt = i;
                break;
            end
        end

        if (insertAt == -1) then
            self:AddItem(effectItem);
        else
            self:InsertItem(insertAt, effectItem);
        end
    elseif (self.settings.sortMethod == EffectWindow.SortCategory) then
        local category = effect:GetCategory();

        local i = 0;
        local insertAt = -1;

        for i = 1, self:GetItemCount() do
            local testEffect = self:GetItem(i):GetEffect();
            local testEffectCategory = testEffect:GetCategory();
            local dif = testEffectCategory - category;
            if (not self.settings.sortAscending) then
                dif = dif * -1;
            end
            if (self.settings.fillBottomToTop) then
                dif = dif * -1;
            end
            
            if (dif >= 0) then
                insertAt = i;
                break;
            end
        end

        if (insertAt == -1) then
            self:AddItem(effectItem);
        else
            self:InsertItem(insertAt, effectItem);
        end
    else
        -- no sorting
        self:AddItem(effectItem);
    end
end

function EffectList:EffectRemoved(effect)
    local i;

    for i = 1, self:GetItemCount() do
        local effectListItem = self:GetItem(i):GetEffect();

        if (effect == effectListItem) then
            local effectElement = self:GetItem(i);
            effectElement:SetVisible(false);
            self:RemoveItemAt(i);
            break;
        end
    end
end
