import "Turbine.Utils";
import "Turbine.Gameplay";
import "PengorosPlugins.BuffBars";

EffectSettings = class();

function EffectSettings:Constructor(saved)
    if (saved ~= nil) then
        for i,v in pairs(saved) do
            self[i] = v;
        end
    else
        self:SetDefaults();
    end
end

function EffectSettings:Copy()
    -- deep copy table function
    local function deepcopy(object)
        local lookup_table = {}
        local function _copy(object)
            if type(object) ~= "table" then
                return object
            elseif lookup_table[object] then
                return lookup_table[object]
            end
            local new_table = {}
            lookup_table[object] = new_table
            for index, value in pairs(object) do
                new_table[_copy(index)] = _copy(value)
            end
            return setmetatable(new_table, getmetatable(object))
        end
        return _copy(object)
    end
    
    return deepcopy(self);
end

function EffectSettings:SetDefaults()
    -- effect bars
    self.enabled = true;
    self.name = "BuffBar Window"
    self.posX = 10;
    self.posY = 200;
    self.barHeight = 24;
    self.barWidth = 200;
    self.showDebuffBar = true;
    self.showBuffBar = true;
    self.showCoolDownBar = true;
    self.visibleType = EffectWindow.VisibleAlways; -- v4
    self.barOrder = EffectWindow.OrderDebuffBuffCoolDown;
    self.fillBottomToTop = false;

    -- sort options
    self.sortMethod = EffectWindow.SortTime;
    self.sortAscending = true;
    
    -- flash options
    self.allowFlashing = true;
    self.flashTime = 15;

    -- effect category filters
    self.showDisease = true;
    self.showFear = true;
    self.showPoison = true;
    self.showWound= true;
    self.showCorruption = true;
    self.showCry = true;
    self.showElemental = true;
    self.showPhysical = true;
    self.showSong = true;
    self.showUndefined = true;
    
    -- cooldown category filters
    self.showDiseaseCD = true; -- v4
    self.showFearCD = true; -- v4
    self.showPoisonCD = true; -- v4
    self.showWoundCD = true; -- v4
    self.showMoraleCD = true; -- v4
    self.showPowerCD = true; -- v4

    -- effect type filters
    self.showCurableOnly = false;
    self.ignored = {};
    self.override = {};
    
    -- effect duration filters
    self.minDurationFilter = false; -- v4
    self.minDuration = 5; -- v4
    self.maxDurationFilter = false; -- v4
    self.maxDuration = 300; -- v4

    -- effect ui
    self.showEffectIcon = true;
    self.showEffectName = true;
    self.showEffectTimer = true;
    self.showEffectDuration = true; -- v4
    self.backgroundColor = {a=128, r=0, g=0, b=0};
    self.font = Turbine.UI.Lotro.Font.Verdana12;
    
    -- colors
    self.colors = {};
    local col = Settings.Flash;
    self.colors[col] = {};
    self.colors[col].remaining = {a=255, r=255, g=255, b=255};
    self.colors[col].elapsed = {a=102, r=255, g=255, b=255};
    
    col = Settings.UndefinedEffect;
    self.colors[col] = {};
    self.colors[col].remaining = {a=255, r=0, g=121, b=254};
    self.colors[col].elapsed = {a=102, r=0, g=121, b=254};
    
    col = Settings.DiseaseEffect;
    self.colors[col] = {};
    self.colors[col].remaining = {a=255, r=232, g=234, b=0};
    self.colors[col].elapsed = {a=102, r=232, g=234, b=0};
    
    col = Settings.FearEffect;
    self.colors[col] = {};
    self.colors[col].remaining = {a=255, r=215, g=28, b=227};
    self.colors[col].elapsed = {a=102, r=215, g=28, b=227};

    col = Settings.PoisonEffect;
    self.colors[col] = {};
    self.colors[col].remaining = {a=255, r=64, g=226, b=28};
    self.colors[col].elapsed = {a=102, r=64, g=226, b=28};

    col = Settings.WoundEffect;
    self.colors[col] = {};
    self.colors[col].remaining = {a=255, r=255, g=29, b=29};
    self.colors[col].elapsed = {a=102, r=255, g=29, b=29};

    col = Settings.PhysicalEffect;
    self.colors[col] = {};
    self.colors[col].remaining = {a=255, r=0, g=121, b=254};
    self.colors[col].elapsed = {a=102, r=0, g=121, b=254};

    col = Settings.ElementalEffect;
    self.colors[col] = {};
    self.colors[col].remaining = {a=255, r=0, g=121, b=254};
    self.colors[col].elapsed = {a=102, r=0, g=121, b=254};

    col = Settings.SongEffect;
    self.colors[col] = {};
    self.colors[col].remaining = {a=255, r=0, g=121, b=254};
    self.colors[col].elapsed = {a=102, r=0, g=121, b=254};

    col = Settings.CorruptionEffect;
    self.colors[col] = {};
    self.colors[col].remaining = {a=255, r=0, g=121, b=254};
    self.colors[col].elapsed = {a=102, r=0, g=121, b=254};

    col = Settings.CryEffect;
    self.colors[col] = {};
    self.colors[col].remaining = {a=255, r=0, g=121, b=254};
    self.colors[col].elapsed = {a=102, r=0, g=121, b=254};
    
    col = Settings.MoraleCoolDown;
    self.colors[col] = {};
    self.colors[col].remaining = {a=255, r=0, g=255, b=29};
    self.colors[col].elapsed = {a=102, r=0, g=255, b=29};
    
    col = Settings.PowerCoolDown;
    self.colors[col] = {};
    self.colors[col].remaining = {a=255, r=0, g=29, b=255};
    self.colors[col].elapsed = {a=102, r=0, g=29, b=255};
    
    col = Settings.DiseaseCoolDown;
    self.colors[col] = {};
    self.colors[col].remaining = {a=255, r=232, g=234, b=0};
    self.colors[col].elapsed = {a=102, r=232, g=234, b=0};
    
    col = Settings.FearCoolDown;
    self.colors[col] = {};
    self.colors[col].remaining = {a=255, r=215, g=28, b=227};
    self.colors[col].elapsed = {a=102, r=215, g=28, b=227};

    col = Settings.PoisonCoolDown;
    self.colors[col] = {};
    self.colors[col].remaining = {a=255, r=64, g=226, b=28};
    self.colors[col].elapsed = {a=102, r=64, g=226, b=28};

    col = Settings.WoundCoolDown;
    self.colors[col] = {};
    self.colors[col].remaining = {a=255, r=255, g=29, b=29};
    self.colors[col].elapsed = {a=102, r=255, g=29, b=29};
end

function EffectSettings:GetBackColor()
    return self:GetColor(self.backgroundColor);
end

function EffectSettings:GetRemainingColor(category)
    return self:GetColor(self.colors[category].remaining);
end

function EffectSettings:GetElapsedColor(category)
    return self:GetColor(self.colors[category].elapsed);
end

function EffectSettings:GetColor(color)
    return Turbine.UI.Color(color.a / 255, color.r / 255, color.g / 255, color.b / 255);
end

-- returns a copy of the color array
function EffectSettings:GetRemainingColorArray(category)
    local c = self.colors[category].remaining;
    return {a=c.a, r=c.r, g=c.g, b=c.b};
end

function EffectSettings:SetRemainingColorArray(category, color)
    self.colors[category]["remaining"] = color;
end

-- returns a copy of the color array
function EffectSettings:GetElapsedColorArray(category)
    local c = self.colors[category].elapsed;
    return {a=c.a, r=c.r, g=c.g, b=c.b};
end

function EffectSettings:SetElapsedColorArray(category, color)
    self.colors[category]["elapsed"] = color;
end

function EffectSettings:ResetIgnored()
    self.ignored = {};
end

function EffectSettings:AddIgnoredEffect(effect)
    self.ignored[effect] = true;
end

function EffectSettings:IsIgnoredEffect(effect)
    return self.ignored[effect];
end

function EffectSettings:ResetOverride()
    self.override = {};
end

function EffectSettings:AddOverrideEffect(effect)
    self.override[effect] = true;
end

function EffectSettings:IsOverrideEffect(effect)
    return self.override[effect];
end
