import "Turbine.Gameplay";
import "Turbine.Utils";
import "PengorosPlugins.BuffBars";

Log = class();

function Log:Constructor(settings)
    self.settings = settings;

    -- add listeners
    effectManager:AddListener(self);
end

function Log:Destroy()
    effectManager:RemoveListener(self);
end

function Log:EffectAdded(effect)
    local debuff = effect:IsDebuff();
    if (debuff and (not self.settings.showDebuffs)) then return
    elseif ((not debuff) and (not self.settings.showBuffs)) then return
    end
    
    local log = "Effect Added: ";
    log = log .. self:FormatEffect(effect, false);
    Turbine.Shell.WriteLine(log);
    
    --[[
    <rgb=#FF0000>Red Text</rgb>
    <rgb=#00FF00>Green Text</rgb>
    <rgb=#0000FF>Blue Text</rgb>
    --]]
end

function Log:EffectRemoved(effect)
    local debuff = effect:IsDebuff();
    if (debuff and (not self.settings.showDebuffs)) then return
    elseif ((not debuff) and (not self.settings.showBuffs)) then return
    end
    
    local log = "Effect Removed: ";
    log = log .. self:FormatEffect(effect, true);
    Turbine.Shell.WriteLine(log);
end

function Log:FormatEffect(effect, removed)
    local log = "";
    if (self.settings.logId) then
        log = log .. ", id=" .. effect:GetID();
    end
    if (self.settings.logName) then
        log = log .. ", name=" .. effect:GetName();
    end
    if (self.settings.logDescription) then
        local desc = effect:GetDescription();
        if (desc == nil or string.len(desc) == 0) then
            desc = "None";
        end
        log = log .. ", description=" .. desc;
    end
    if (self.settings.logCategory) then
        local category = effect:GetCategory();
        local catName = "Undefined";
        
        if (category == Turbine.Gameplay.EffectCategory.Corruption) then
            catName = "Corruption";
        elseif (category == Turbine.Gameplay.EffectCategory.Cry) then
            catName = "Cry";
        elseif (category == Turbine.Gameplay.EffectCategory.Disease) then
            catName = "Disease";
        elseif (category == Turbine.Gameplay.EffectCategory.Elemental) then
            catName = "Elemental";
        elseif (category == Turbine.Gameplay.EffectCategory.Fear) then
            catName = "Fear";
        elseif (category == Turbine.Gameplay.EffectCategory.Physical) then
            catName = "Physical";
        elseif (category == Turbine.Gameplay.EffectCategory.Poison) then
            catName = "Poison";
        elseif (category == Turbine.Gameplay.EffectCategory.Song) then
            catName = "Song";
        elseif (category == Turbine.Gameplay.EffectCategory.Wound) then
            catName = "Wound";
        end
        
        log = log .. ", category=" .. catName;
    end
    if (self.settings.logDuration) then
        local duration = effect:GetDuration();
        if (removed) then
            local gameTime = Turbine.Engine.GetGameTime();
            duration = math.ceil(gameTime - effect:GetStartTime());
        end
        log = log .. ", duration=" .. self:FormatTime(duration);
    end
    if (self.settings.logCurable) then
        log = log .. ", curable=" .. tostring(effect:IsCurable());
    end
    if (self.settings.logDebuff) then
        log = log .. ", debuff=" .. tostring(effect:IsDebuff());
    end
    
    if (string.len(log) > 0) then
        log = string.sub(log, 3);
    end
    
    return log;
end

function Log:FormatTime(time)
    local temp = time;
    local sec = temp % 60;
    temp = temp - sec;
    temp = temp / 60;
    local min = temp % 60;
    local filler = sec < 10 and "0" or "";
    temp = temp - min;
    temp = temp / 60;
    local hour = temp % 24;
    if (hour > 0) then
        local filler2 = min < 10 and "0" or "";
        return hour .. ":" .. filler2 .. min .. ":" .. filler .. sec;
    else
        return min .. ":" .. filler .. sec;
    end
end
