import "Turbine.UI";
import "Turbine.UI.Lotro";
import "PengorosPlugins.BuffBars";
-- import the dragbar control if it is available
local importDragBar = function()
    import "Deusdictum.UI";
end
hasDragBar = pcall(importDragBar);

--[[
-- bug submission code
window = Turbine.UI.Window();
window:SetPosition(0, 0);
window:SetSize(0, 0);
window:SetBackColor(Turbine.UI.Color(1, 0, 0, 0));
window.MouseClick = function(sender, args)
    Turbine.Shell.WriteLine("click");
end
window:SetVisible(false);
window:SetVisible(true);
window:SetSize(Turbine.UI.Display:GetWidth(), Turbine.UI.Display:GetHeight());
--]]

-- load the plugin pluginSettings
pluginSettings = Settings();

-- plugin managers
effectManager = EffectManager();
coolDownManager = CoolDownManager();
playerManager = PlayerManager();
keyManager = KeyManager();

-- plugin windows
local effectWindows = {};
local quickslotBars = {};
local effectLogger = nil;

-- reset windows
function ResetState()
    local i;
    -- destroy the existing effect windows
    for i = 1, table.getn(effectWindows) do
        effectWindows[i]:Destroy();
        effectWindows[i] = nil;
    end
    effectWindows = nil;
    
    -- destroy existing quick slots
    for i = 1, table.getn(quickslotBars) do
        quickslotBars[i]:Destroy();
        quickslotBars[i] = nil;
    end
    quickslotBars = nil;
    
    -- destroy the existing logger
    if (effectLogger ~= nil) then
        effectLogger:Destroy();
        effectLogger = nil;
    end

    -- create new effect windows    
    effectWindows = {};
    for i = 1, table.getn(pluginSettings.windows) do
        if (pluginSettings.windows[i].enabled) then
            table.insert(effectWindows, EffectWindow(pluginSettings.windows[i]));
        end
    end

    -- create new quickslot bars    
    quickslotBars = {};
    for i = 1, table.getn(pluginSettings.quickslots) do
        if (pluginSettings.quickslots[i].enabled) then
            table.insert(quickslotBars, QuickslotBar(pluginSettings.quickslots[i]));
        end
    end
    
    -- effect logger
    if (pluginSettings.log.enabled) then
        effectLogger = Log(pluginSettings.log);
    end
end

-- show options
function ShowOptions()
    if (optionsWindow == nil) then
        optionsWindow = OptionsWindow();
    end
end

-- logging
function EnableLogging(enable)
    if (pluginSettings.log.enabled ~= enable) then
        pluginSettings.log.enabled = enable;
        if (enable) then
            Turbine.Shell.WriteLine("Effect Logging Enabled");
            effectLogger = Log(pluginSettings.log);
        else
            Turbine.Shell.WriteLine("Effect Logging Disabled");
            effectLogger:Destroy();
            effectLogger = nil;
        end
    end
end

-- commands
buffBarsCommand = Turbine.ShellCommand();
Turbine.Shell.AddCommand("buffbars", buffBarsCommand);
function buffBarsCommand:GetShortHelp()
    return "usage: /buffbars setup | logging <enable|disable>";
end
function buffBarsCommand:Execute(cmd, args)
    if (args == "setup") then
        ShowOptions();
    elseif (args == "logging enable") then
        EnableLogging(true);
    elseif (args == "logging disable") then
        EnableLogging(false);
    else
        Turbine.Shell.WriteLine(self:GetShortHelp());
    end
end

-- plugin unload event
function UnloadPlugin()
    local i;
    for i = 1, table.getn(effectWindows) do
        effectWindows[i]:Destroy();
        effectWindows[i] = nil;
    end
    for i = 1, table.getn(quickslotBars) do
        quickslotBars[i]:Destroy();
        quickslotBars[i] = nil;
    end
    if (effectLogger ~= nil) then
        effectLogger:Destroy();
        effectLogger = nil;
    end
    if (optionsWindow ~= nil) then
        optionsWindow:Destroy();
    end
    pluginSettings:Save();
end

-- constrain the window to the display
function validatePosition(x, y, width, height)
    x = math.min(math.max(x, 0), Turbine.UI.Display:GetWidth() - width);
    y = math.min(math.max(y, 0), Turbine.UI.Display:GetHeight() - height);
    return x, y;
end

-- set the initial state
ResetState();

Turbine.Shell.WriteLine("BuffBars v1.0 by Pengoros");
