import "Turbine.Gameplay";
import "Turbine.UI";
import "Turbine.Utils";
import "PengorosPlugins.BuffBars";
import "PengorosPlugins.UI";

--OptionsWindow = class(Turbine.UI.Lotro.Window);
OptionsWindow = class(PengorosPlugins.UI.Window);

function OptionsWindow:Constructor()
    --Turbine.UI.Lotro.Window.Constructor(self);
    PengorosPlugins.UI.Window.Constructor(self);
    
    self:SetText("BuffBars Options");
    self:SetSize(575, 535);
    self.Closed = function(sender, args)
        self.effectsPanel:Closed();
        
        -- save the position
        pluginSettings.optionsX, pluginSettings.optionsY = self:GetPosition();
        pluginSettings:Save();
        
        -- remove the ref
        self:Destroy();
        optionsWindow = nil;
    end
    
    -- restore the position if there is one
    if (pluginSettings.optionsX ~= nil) then
        self:SetPosition(validatePosition(pluginSettings.optionsX, pluginSettings.optionsY, self:GetWidth(), self:GetHeight()));
    end
    
    -- listen for events
    keyManager:AddListener(self);

    self.sideBar = Turbine.UI.Control();
    self.sideBar:SetParent(self);
    self.sideBar:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend);
    self.sideBar:SetBackground("PengorosPlugins/UI/Resources/options_menu_side_bar.tga");
    self.sideBar:SetSize(140, 507);
    self.sideBar:SetPosition(432, 24);
    self.sideBar:SetMouseVisible(false);
    
    self.quickslotButton = PengorosPlugins.UI.Button();
    self.quickslotButton:SetParent(self);
    self.quickslotButton:SetPosition(439, 85);
    self.quickslotButton:SetSize(127, 20);
    self.quickslotButton:SetText("Quickslot");
    self.quickslotButton.Click = function(sender, args)
        self:SwitchPanel(self.quickslotPanel, self.quickslotButton);
    end

    self.effectsButton = PengorosPlugins.UI.Button();
    self.effectsButton:SetParent(self);
    self.effectsButton:SetPosition(439, 110);
    self.effectsButton:SetSize(127, 20);
    self.effectsButton:SetText("Effects");
    self.effectsButton.Click = function(sender, args)
        self:SwitchPanel(self.effectsPanel, self.effectsButton);
    end

    self.loggingButton = PengorosPlugins.UI.Button();
    self.loggingButton:SetParent(self);
    self.loggingButton:SetPosition(439, 135);
    self.loggingButton:SetSize(127, 20);
    self.loggingButton:SetText("Logging");
    self.loggingButton.Click = function(sender, args)
        self:SwitchPanel(self.loggingPanel, self.loggingButton);
    end

    self.cancelButton = Turbine.UI.Lotro.Button();
    self.cancelButton:SetParent(self);
    self.cancelButton:SetPosition(122, 502);
    self.cancelButton:SetSize(100, 20);
    self.cancelButton:SetText("Cancel");
    self.cancelButton.Click = function(sender, args)
        self:Close();
    end

    self.acceptButton  = Turbine.UI.Lotro.Button();
    self.acceptButton:SetParent(self);
    self.acceptButton:SetPosition(237, 502);
    self.acceptButton:SetSize(100, 20);
    self.acceptButton:SetText("Accept");
    self.acceptButton.Click = function(sender, args)
        self:SaveOptions();
        self:Close();
        ResetState();
        pluginSettings:Save();
    end

    self.scrollBar = Turbine.UI.Lotro.ScrollBar();
    self.scrollBar:SetOrientation(Turbine.UI.Orientation.Vertical);
    self.scrollBar:SetParent(self);
    self.scrollBar:SetSize(10, 447);
    self.scrollBar:SetPosition(421, 53);

    self.containerListBox = Turbine.UI.ListBox();
    self.containerListBox:SetParent(self);
    self.containerListBox:SetOrientation(Turbine.UI.Orientation.Horizontal);
    self.containerListBox:SetVerticalScrollBar(self.scrollBar);
    self.containerListBox:SetSize(380, 447);
    self.containerListBox:SetPosition(36, 53);
    
    self.quickslotPanel = OptionsQuickslotPanel();
    self.quickslotPanel:SetVisible(false);

    self.effectsPanel = OptionsEffectPanel();
    self.effectsPanel:SetVisible(false);
    
    self.loggingPanel = OptionsLogPanel();
    self.loggingPanel:SetVisible(false);
    
    -- show the quickslot panel        
    self.quickslotPanel:SetVisible(true);
    self.containerListBox:AddItem(self.quickslotPanel);
    self.quickslotButton:SetSelected(true);

    -- load the options
    self:SetOptions();
end

function OptionsWindow:SwitchPanel(panel, button)
    if (not self.containerListBox:ContainsItem(panel)) then
        local oldPanel = self.containerListBox:GetItem(1);
        
        panel:SetVisible(true);
        oldPanel:SetVisible(false);
    
        self.containerListBox:RemoveItem(oldPanel);
        self.containerListBox:AddItem(panel);

        self.quickslotButton:SetSelected(button == self.quickslotButton);
        self.effectsButton:SetSelected(button == self.effectsButton);
        self.loggingButton:SetSelected(button == self.loggingButton);
    end
end

function OptionsWindow:UiHidden(hidden)
    self:SetVisible(not hidden);
end

function OptionsWindow:SetOptions()
    self.quickslotPanel:SetOptions();
    self.effectsPanel:SetOptions();
    self.loggingPanel:SetOptions();
end

function OptionsWindow:SaveOptions()
    self.quickslotPanel:SaveOptions();
    self.effectsPanel:SaveOptions();
    self.loggingPanel:SaveOptions();
end

function OptionsWindow:Destroy()
    self:SetVisible(false);
    keyManager:RemoveListener(self);
    self.quickslotPanel:Destroy();
end
