import "Turbine.Utils";
import "Turbine.Gameplay";
import "PengorosPlugins.BuffBars";

QuickslotSettings = class();

function QuickslotSettings:Constructor(saved)
    if (saved ~= nil) then
        for i,v in pairs(saved) do
            self[i] = v;
        end
    else
        self:SetDefaults();
    end
end

function QuickslotSettings:Copy()
    -- deep copy table function
    local function deepcopy(object)
        local lookup_table = {}
        local function _copy(object)
            if type(object) ~= "table" then
                return object
            elseif lookup_table[object] then
                return lookup_table[object]
            end
            local new_table = {}
            lookup_table[object] = new_table
            for index, value in pairs(object) do
                new_table[_copy(index)] = _copy(value)
            end
            return setmetatable(new_table, getmetatable(object))
        end
        return _copy(object)
    end
    
    return deepcopy(self);
end

function QuickslotSettings:SetDefaults()
    local displayWidth = Turbine.UI.Display:GetWidth();
    local displayHeight = Turbine.UI.Display:GetHeight();
    
    self.enabled = true;
    self.name = "Quickslot Bar";
    self.visibleType = EffectWindow.VisibleAlways; -- v4
    self.quickslotSize = 36;
    self.quickslotSpacing = 5;
    self.qsX = displayWidth / 2;
    self.qsY = ((displayHeight - 300) / 2) + 300;
    self.moralePercent = .30;
    self.powerPercent = .30;
    
    -- filters
    self.showDisease = true;
    self.showFear = true;
    self.showPoison = true;
    self.showWound = true;
    self.showMorale = true;
    self.showPower = true;
end
